'use strict';
var __awaiter =
  (this && this.__awaiter) ||
  function (thisArg, _arguments, P, generator) {
    function adopt(value) {
      return value instanceof P
        ? value
        : new P(function (resolve) {
            resolve(value);
          });
    }
    return new (P || (P = Promise))(function (resolve, reject) {
      function fulfilled(value) {
        try {
          step(generator.next(value));
        } catch (e) {
          reject(e);
        }
      }
      function rejected(value) {
        try {
          step(generator['throw'](value));
        } catch (e) {
          reject(e);
        }
      }
      function step(result) {
        result.done
          ? resolve(result.value)
          : adopt(result.value).then(fulfilled, rejected);
      }
      step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
  };
Object.defineProperty(exports, '__esModule', { value: true });
exports.CharacterCreator = void 0;
const ClassService_1 = require('../services/classes/ClassService');
const RaceService_1 = require('../services/races/RaceService');
const SpellService_1 = require('../services/spells/SpellService');
const Character_1 = require('./Character');
class CharacterCreator {
  constructor() {
    this.spellService = new SpellService_1.SpellService();
    this.classService = new ClassService_1.ClassService();
    this.raceService = new RaceService_1.RaceService();
  }
  getRandomCharacter(numberOfSpells = 4, numberOfFeatures = 2) {
    return __awaiter(this, void 0, void 0, function* () {
      const race = yield this.raceService.getRandom();
      const classData = yield this.classService.getRandom();
      const spells = yield this.getRandomSpells(
        classData.index,
        numberOfSpells,
      );
      const features = yield this.getRandomFeatures(
        classData.index,
        numberOfFeatures,
      );
      const randomSubclass =
        classData.subclasses[
          Math.floor(Math.random() * classData.subclasses.length)
        ];
      const charData = {
        race,
        class: classData,
        subclass: randomSubclass,
        spells,
        proficiencies: classData.proficiencies,
        features,
      };
      return new Character_1.Character(charData);
    });
  }
  getRandomSpells(classIndex, numberOfSpells = 4) {
    return __awaiter(this, void 0, void 0, function* () {
      const spells = yield this.spellService.getClassSpells(classIndex);
      const randomSpells = {
        count: 0,
        results: [],
      };
      const amountOfSpells =
        spells.count > numberOfSpells ? numberOfSpells : spells.count;
      for (let i = 0; i < amountOfSpells; i++) {
        const random = Math.floor(Math.random() * spells.count);
        const spell = spells.results[random];
        if (spell) {
          randomSpells.results.push(spell);
          randomSpells.count++;
        }
      }
      return randomSpells;
    });
  }
  getRandomFeatures(classIndex, numberOfFeatures = 4) {
    return __awaiter(this, void 0, void 0, function* () {
      const features = yield this.classService.getClassFeatures(classIndex);
      const randomFeatures = {
        count: 0,
        results: [],
      };
      const amountOfFeatures =
        features.count > numberOfFeatures ? numberOfFeatures : features.count;
      for (let i = 0; i < amountOfFeatures; i++) {
        const random = Math.floor(Math.random() * features.count);
        const feature = features.results[random];
        if (feature) {
          randomFeatures.results.push(feature);
          randomFeatures.count++;
        }
      }
      return randomFeatures;
    });
  }
}
exports.CharacterCreator = CharacterCreator;
